//Author: LabJack
//March 1, 2007
//Header for U3 example helper functions.

#ifndef _U3_H
#define _U3_H

#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include "labjackusb.h"


typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;


//Structure for storing calibration constants
struct U3_CALIBRATION_INFORMATION {
  uint8 prodID;
  double ainSESlope;
  double ainSEOffset;
  double ainDiffSlope;
  double ainDiffOffset;
  double dacSlope[2];
  double dacOffset[2];
  double tempSlope;
  double tempSlopeLow;
  double calTemp;
  double vref;
  double vref15;
  double vreg;
};

typedef struct U3_CALIBRATION_INFORMATION u3CalibrationInfo;

// Structure for U3 hardware information

struct U3_DEVICE_INFO {
	float HardwareVersion; 
	long SerialNumber; 
	long ProductID; 
	float FirmwareVersion; 
	float BootloaderVersion;
	long LocalID;
};

typedef struct U3_DEVICE_INFO U3DeviceInfo;

/* Functions */

void normalChecksum( uint8 *b,
                     int n);
//Adds checksum to a data packet for normal command format.
//b = data packet for normal command
//n = size of data packet

void extendedChecksum( uint8 *b,
                       int n);
//Adds checksum to a data packet for extended command format.
//b = data packet for extended command
//n = size of data packet

uint8 normalChecksum8( uint8 *b,
                       int n);
//Returns the Checksum8 for a normal command data packet.
//b = data packet for normal command
//n = size of data packet

uint16 extendedChecksum16( uint8 *b,
                           int n);
//Returns the Checksum16 for a extended command data packet.
//b = data packet for extended command
//n = size of data packet

uint8 extendedChecksum8( uint8 *b);
//Returns the Checksum8 for a extended command data packet.
//b = data packet for extended command

HANDLE openUSBConnection( int localID);
//Opens a U3 with the given localID.  Returns NULL on failure, or a HANDLE
//on success.

void closeUSBConnection( HANDLE hDevice);
//Closes a HANDLE to a U3 device.

long getTickCount();
//Returns the number of milliseconds that has elasped since the system was
//started.

long getCalibrationInfo( HANDLE hDevice,
                         u3CalibrationInfo *caliInfo);
//Gets calibration information from memory blocks 0-2 of a U3.  Returns the
//calibration information in a calibrationInfo structure.
//hDevice = handle to a U3 device
//caliInfo = structure where calibrarion information will be stored

double FPuint8ArrayToFPDouble( uint8 *buffer,
                               int startIndex);
//Converts a fixed point byte array (starting a startIndex) to a floating point
//double value.  This function is used primarily by getCalibrationInfo.

long isCalibrationInfoValid(u3CalibrationInfo *caliInfo);
//Performs a simple check to determine if the caliInfo struct was set up by
//getCalibrationInfo.  Returns 0 if caliInfo is not valid, or 1 if it is.
//caliInfo = structure where calibrarion information is stored

long binaryToCalibratedAnalogVoltage( u3CalibrationInfo *caliInfo,
                                      int dac1Enabled,
                                      uint8 negChannel,
                                      uint16 bytesVoltage,
                                      double *analogVoltage);
//Translates the binary analog input bytes read from the U3, to a voltage value
//(calibrated).  Call getCalibrationInfo first to set up caliInfo.  Returns -1
//on error, 0 on success.
//caliInfo = structure where calibrarion information is stored
//dac1Enabled = If this is nonzero (True), then it is indicated that DAC1 is
//             enabled and analog voltage will be calculated with Vreg.  If
//             this is 0 (False), then it is indicated that DAC1 is disabled
//             and the analog voltage will be calculated with the AIN slopes
//             and offsets. 
//negChannel = the negative channel of the differential analog reading
//bytesVoltage = the 2 byte voltage that will be converted to a analog value
//analogVoltage = the converted analog voltage

long analogToCalibratedBinaryVoltage( u3CalibrationInfo *caliInfo,
                                      int DACNumber,
                                      double analogVoltage,
                                      uint16 *bytesVoltage);
//Translates a voltage value to binary analog input bytes (calibrated) that can
//be sent to a U3.  Call getCalibrationInfo first to set up caliInfo.  Returns
//-1 on error, 0 on success.
//caliInfo = structure where calibrarion information is stored
//DACNumber - channel number of the DAC
//analogVoltage = the analog voltage that will be converted to a byte value
//bytesVoltage = the converted byte voltage

long binaryToCalibratedAnalogTemperature( u3CalibrationInfo *caliInfo,
                                          uint16 bytesTemperature,
                                          double *analogTemperature);
//Translates the binary analog bytes read from the U3, to a Kelvin temperature
//value (calibrated).  Call getCalibrationInfo first to set up caliInfo.
//Returns -1 on error, 0 on success.
//caliInfo = structure where calibrarion information is stored
//bytesTemperature = the 2 byte temperature that will be converted to an analog value
//analogTemperature = the converted analog temperature

long binaryToUncalibratedAnalogVoltage( int dac1Enabled, 
                                        uint8 negChannel,
                                        uint16 bytesVoltage,
                                        double *analogVoltage);
//Translates the binary analog input bytes read from the u3, to a voltage value
//(uncalibrated). Returns -1 on error, 0 on success.
//dac1Enabled = If this is nonzero (True), then it is indicated that DAC1 is
//             enabled and analog voltage will be calculated with Vreg.  If
//             this is 0 (False), then it is indicated that DAC1 is disabled
//             and the analog voltage will be calculated with the AIN slopes
//             and offsets.
//negChannel = the negative channel of the differential analog reading
//bytesVoltage = the 2 byte voltage that will be converted to a analog value
//analogVoltage = the converted analog voltage

long analogToUncalibratedBinaryVoltage( double analogVoltage,
                                        uint16 *bytesVoltage);
//Translates a voltage value to binary analog input bytes (uncalibrated) that
//can be sent to a U3.  Returns -1 on error, 0 on success. */
//analogVoltage = the analog voltage that will be converted to a byte value
//bytesVoltage = the converted byte voltage

long binaryToUncalibratedAnalogTemperature( uint16 bytesTemperature,
                                            double *analogTemperature);
//Translates the binary analog bytes read from the U3, to a Kelvin temperature
//value (uncalibrated).  Call getCalibrationInfo first to set up caliInfo.
//Returns -1 on error, 0 on success.
//bytesTemperature = the 2 byte temperature that will be converted to an analog value
//analogTemperature = the converted analog temperature


/* Easy Functions (Similar to the easy functions in the UD driver for Windows) */

long eAIN( HANDLE Handle,
           u3CalibrationInfo *CalibrationInfo,
           long ConfigIO,
           long *DAC1Enable,
           long ChannelP,
           long ChannelN,
           double *Voltage,
           long Range,
           long Resolution,
           long Settling,
           long Binary,
           long Reserved1,
           long Reserved2);
//An "easy" function that returns a reading from one analog input.  This function does
//not automatically configure the specified channels as analog input, unless ConfigIO
//is set as True.  Returns 0 for no error, or -1 or >0 value (low-level errorcode) on error.
//Handle = Handle to a U3 device.
//CalibrationInfo = Structure where calibration information is stored.
//ConfigIO = If this is nonzero (True), then 1 or 2 ConfigIO low-level function calls
//           will be made in addition to the 1 Feedback call to set the specified Channels
//           to analog inputs.  If this is 0 (False), then only a Feedback low-level call
//           will be made, and an error will be returned if the specified Channels are not
//           already set to analog inputs.
//DAC1Enable = This parameter helps to determine the appropriate equation to use when
//             calculating Voltage.  If the long variable that is being pointed to is
//             nonzero (True), then it is indicated that DAC1 is enabled.  If it is 0
//             (False), then it is indicated that DAC1 is disabled.
//             For both case, if ConfigIO is set to True, then input value will be ignored
//             and the output value will be set to the current DAC1Enable value in the
//             ConfigIO low-level response.
//ChannelP = The positive AIN channel to acquire.
//ChannelN = The negative AIN channel to acquire.  For single-ended channels on the U3,
//           this parameter should be 31 (see Section 2.6.1).
//Voltage = Returns the analog input reading, which is generally a voltage.
//Range = Ignored on the U3.
//Resolution = Pass a nonzero value to enable QuickSample.
//Settling = Pass a nonzero value to enable LongSettling.
//Binary = If this is nonzero (True), the Voltage parameter will return the raw binary
//         value.
//Reserved (1&2) = Pass 0.


long eDAC( HANDLE Handle,
           u3CalibrationInfo *CalibrationInfo,
           long ConfigIO,
           long Channel,
           double Voltage,
           long Binary,
           long Reserved1,
           long Reserved2);
//An "easy" function that writes a value to one analog output.  This function does not
//automatically enable the specified analog output, unless ConfigIO is set as True.
//Returns 0 for no error, or -1 or >0 value (low-level errorcode) on error.
//Handle = Handle to a U3 device.
//CalibrationInfo = structure where calibrarion information is stored
//ConfigIO = If this is nonzero (True) and Channel is 1, then 1 ConfigIO low-level function
//           call will be made in addition to the 1 Feedback call to enable DAC1.  If this is
//           0 (False), then only a Feedback low-level call will be made, and an error will
//           be returned if DAC1 is not already enabled.
//Channel = The analog output channel to write to.
//Voltage = The voltage to write to the analog output.
//Binary = If this is nonzero (True), the value passed for Voltage should be binary.
//Reserved (1&2) = Pass 0.

long eDI( HANDLE Handle,
          long ConfigIO,
          long Channel,
          long *State);
//An "easy" function that reads the state of one digital input.  This function does
//not automatically configure the specified channel as digital, unless ConfigIO is
//set as True.  Returns 0 for no error, or -1 or >0 value (low-level errorcode) on error.
//Handle = Handle to a U3 device.
//ConfigIO = If this is nonzero (True), then 2 ConfigIO low-level functions calls will
//           be made in addition to the 1 Feedback call to set Channel as digital.  If
//           this is 0 (False), then only a Feedback low-level call will be made, and an
//           error will be returned if Channel is not already set as digital.
//Channel = The channel to read.  0-19 corresponds to FIO0-CIO3.
//State = Returns the state of the digital input.  0=False=Low and 1=True=High.

long eDO( HANDLE Handle,
          long ConfigIO,
          long Channel,
          long State);
//An "easy" function that writes the state of one digital output.  This function does
//not automatically configure the specified channel as digital, unless ConfigIO is
//set as True.  Returns 0 for no error, or -1 or >0 value (low-level errorcode) on error.
//Handle = Handle to a U3 device.
//ConfigIO = If this is nonzero (True), then 2 ConfigIO low-level functions calls will
//           be made in addition to the 1 Feedback call to set Channel as digital. If
//           this is 0 (False), then only a Feedback low-level call will be made, and an
//           error will be returned if Channel is not already set as digital.
//Channel = The channel to write to.  0-19 corresponds to FIO0-CIO3.
//State = The state to write to the digital output.  0=False=Low and 1=True=High.

long eTCConfig( HANDLE Handle,
                long *aEnableTimers,
                long *aEnableCounters,
                long TCPinOffset,
                long TimerClockBaseIndex,
                long TimerClockDivisor,
                long *aTimerModes,
                double *aTimerValues,
                long Reserved1,
                long Reserved2);
//An "easy" function that configures and initializes all the timers and counters.
//When needed, this function automatically configures the needed lines as digital.
//Returns 0 for no error, or -1 or >0 value (low-level errorcode) on error.
//Handle = Handle to a U3 device.
//aEnableTimers = An array where each element specifies whether that timer is enabled.
//                Timers must be enabled in order starting from 0, so for instance,
//                Timer1 cannot be enabled without enabling Timer 0 also.  A nonzero
//                for an array element specifies to enable that timer.  For the U3,
//                this array must always have at least 2 elements.
//aEnableCounters = An array where each element specifies whether that counter is enabled.
//                  Counters do not have to be enabled in order starting from 0, so
//                  Counter1 can be enabled when Counter0 is disabled.  A nonzero value
//                  for an array element specifies to enable that counter.  For the U3,
//                  this array must always have at least 2 elements.
//TCPinOffset = Value from 0-8 specifies where to start assigning timers and counters.
//TimerClockBaseIndex = Pass a constant to set the timer base clock.  The default is
//                      LJ_tc48MHZ.
//TimerClockDivisor = Pass a divisor from 0-255 where 0 is a divisor of 256.
//aTimerModes = An array where each element is a constant specifying the mode for that
//              timer.  For the U3, this array must always have at least 2 elements.
//aTimerValues = An array where each element specifies the initial value for that
//               timer.  For the U3, this array must always have at least 2 elements.
//Reserved (1&2) =  Pass 0.

long eTCValues( HANDLE Handle,
                long *aReadTimers,
                long *aUpdateResetTimers,
                long *aReadCounters,
                long *aResetCounters,
                double *aTimerValues,
                double *aCounterValues,
                long Reserved1,
                long Reserved2);
//An "easy" function that updates and reads all the timers and counters.  Returns 0 for
//no error, or -1 or >0 value (low-level errorcode) on error.
//Handle = Handle to a U3 device.
//aReadTimers = An array where each element specifies whether to read that timer.  A
//              nonzero value for an array element specifies to read that timer.  For
//              the U3, this array must always have at least 2 elements.
//aUpdateResetTimers = An array where each element specifies whether to update/reset
//                     that timers.  A nonzero value for an array element specifies to
//                     update/reset that timer.  For the U3, this array must always
//                     have at least 2 elements.
//aReadCounters = An array where each element specifies whether to read that counter.
//                A nonzero value for an array element specifies to read that counter.
//                For the U3, this array must always have at least 2 elements.
//aResetCounters = An array where each element specifies whether to reset that counter.
//                 A nonzero value for an array element specifies to reset that counter.
//                 For the U3, this array must always have at least 2 elements.
//aTimerValues = Input: An array where each element is the new value for that timer.  Each
//               value is only updated if the appropriate element is set in the
//               aUpdateResetTimers array.
//               Output: An array where each element is the value read from that timer if
//               the appropriate element is set in the aReadTimers array.  If the timer
//               mode set for the timer is Quadrature Input, the value needs to be converted
//               from an unsigned 32-bit integer to a signed 32-bit integer (2’s complement).
//               For the U3, this array must always have at least 2 elements.
//aCounterValues = An array where each element is the value read from that counter if
//                 the appropriate element is set in the aReadTimers array.  For the U3,
//                 this array must always have at least 2 elements.
//Reserved (1&2) = Pass 0.


/* Easy Function Helpers */

long ehConfigIO( HANDLE hDevice,
                 uint8 inWriteMask,
                 uint8 inTimerCounterConfig,
                 uint8 inDAC1Enable,
                 uint8 inFIOAnalog,
                 uint8 inEIOAnalog,
                 uint8 *outTimerCounterConfig,
                 uint8 *outDAC1Enable,
                 uint8 *outFIOAnalog,
                 uint8 *outEIOAnalog);
//Used by the eAIN, eDAC, eDI, eDO and eTCConfig easy functions.  This function takes the ConfigIO
//low-level command and response bytes (not including checksum and command bytes) as its parameter
//and performs a ConfigIO call with the U3.  Returns -1 or errorcode (>1 value) on error, 0
//on success.

long ehConfigTimerClock( HANDLE hDevice,
                         uint8 inTimerClockConfig,
                         uint8 inTimerClockDivisor,
                         uint8 *outTimerClockConfig,
                         uint8 *outTimerClockDivisor);
//Used by the eTCConfig easy function.  This function takes the ConfigTimerClock low-level command
//and response bytes (not including checksum and command bytes) as its parameter and performs a
//ConfigTimerClock call with the U3.  Returns -1 or errorcode (>1 value) on error, 0 on success.

long ehFeedback( HANDLE hDevice,
                 uint8 *inIOTypesDataBuff,
                 long inIOTypesDataSize,
                 uint8 *outErrorcode,
                 uint8 *outErrorFrame,
                 uint8 *outDataBuff,
                 long outDataSize);
//Used by the all of the easy functions.  This function takes the Feedback low-level command and
//response bytes (not including checksum and command bytes) as its parameter and performs a
//Feedback call with the U3.  Returns -1 or errorcode (>1 value) on error, 0 on success.

long ConfigU3 ( HANDLE hDevice, 
				uint8 localID, 
				uint8 Wmask,
				U3DeviceInfo *UnitInfo);
//Basic ConfigU3 low level protocol. At this time it will only be used to retrieve unit info
// and change the Local ID if Wmask is properly set. 

long Reset (HANDLE hdevice, uint8 Hard);
// Reset function - for Hard =1 device performs a hard reset. 

long Buzzer (HANDLE hDevice, long Period, 
			 long Toggles, long Continuous);
// Buzzer function

long LED (HANDLE hDevice, int State);
// Turns LED On and Off


long unitTemp (HANDLE handle, u3CalibrationInfo *CalibrationInfo, double *temp);
// Returns unit temperature in Kelvin

/* Easy function constants */


/* Timer clocks for Hardware Version 1.20 or lower */

// 2 MHz
#define LJ_tc2MHZ 10

// 6 MHz
#define LJ_tc6MHZ 11

// 24 MHz
#define LJ_tc24MHZ 12

// 500/Divisor KHz
#define LJ_tc500KHZ_DIV 13

// 2/Divisor MHz
#define LJ_tc2MHZ_DIV 14

// 6/Divisor MHz
#define LJ_tc6MHZ_DIV 15

// 24/Divisor MHz
#define LJ_tc24MHZ_DIV 16


/* Timer clocks for Hardware Version 1.21 or higher */

// 4 MHz
#define LJ_tc4MHZ 20

// 12 MHz
#define LJ_tc12MHZ 21

// 48 MHz
#define LJ_tc48MHZ 22

// 1/Divisor MHz
#define LJ_tc1MHZ_DIV 23

// 4/Divisor MHz
#define LJ_tc4MHZ_DIV 24

// 12/Divisor MHz
#define LJ_tc12MHZ_DIV 25

// 48/Divisor MHz
#define LJ_tc48MHZ_DIV 26


/* Timer modes */

// 16 bit PWM
#define LJ_tmPWM16 0

// 8 bit PWM
#define LJ_tmPWM8 1

// 32-bit rising to rising edge measurement
#define LJ_tmRISINGEDGES32 2

// 32-bit falling to falling edge measurement
#define LJ_tmFALLINGEDGES32 3

// duty cycle measurement
#define LJ_tmDUTYCYCLE 4

// firmware based rising edge counter
#define LJ_tmFIRMCOUNTER 5

// firmware counter with debounce
#define LJ_tmFIRMCOUNTERDEBOUNCE 6

// frequency output
#define LJ_tmFREQOUT 7

// Quadrature
#define LJ_tmQUAD 8

// stops another timer after n pulses
#define LJ_tmTIMERSTOP 9

// read lower 32-bits of system timer
#define LJ_tmSYSTIMERLOW 10

// read upper 32-bits of system timer
#define LJ_tmSYSTIMERHIGH 11

// 16-bit rising to rising edge measurement
#define LJ_tmRISINGEDGES16 12

// 16-bit falling to falling edge measurement
#define LJ_tmFALLINGEDGES16 13

#endif
